"""
RAISE: RBP Activity Inference from Splicing Events

This module contains core functions for:
- Finding splicing targets
- Constructing splicing regulatory networks
- Estimating RBP activity
"""
__version__ = "0.1.0"
